/*
DarkEnergy - algorithme de simulation pour SimuApplet

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.

*/
import java.io.*;
import java.net.*;
import simu.*;
import java.lang.Math;

/**
 * Algorithme de simulation pour SimuApplet.
 * Ce programme calcule l'age de l'univers et l'volution du facteur d'echelle "a" en fonction du temps "t"
 * Les modeles d'Univers consideres sont ceux de Friedmann-Lemaitre-Robertson-Walker (AgeUnivers)
 */
public class DarkEnergy extends SimuApplet {

	public static final double mpc=3.08567758074e22;
	public static final double gyr=3.1688087814e-17;
	public static final double c=2.99792458e8;
	public static final double twopi=6.283185307179586;
	public static final double a0 = 1;
	double h0,Omega_m0,Omega_rad0,Omega_l0,Omega_k0;
	String fsn1a;
	double stop = 1e-6;
	double dz=1e-3;
	int idat;
	double sum_int,a,t,h,integral,age,courbure,muz,chi2,dl,z,sum1,sum2,sum3,tempmuz,yz;
	double[] zobs,muobs,sigobs;
	int len;
	private Fonction1 f1= new Fonction1(); 
	private Fonction2 f2= new Fonction2(); 
	private double xMin = +0.015, xMax = +1.01;// abcisses
	private double yMin = +34.071, yMax = +44.673;// ordonnes
	
	// constructeur de la classe, avec la dfinition des paramtres de simulation
	// et des options d'affichage.
    
	public DarkEnergy() {
        
	// paramtres d'entre
        ParamIn h0 = new ParamIn();
        h0.type = "nombre";
        h0.label = "h0";
        h0.titre = "Constante de Hubble aujourd'hui";
        h0.unit = "km/s/Mpc";
        h0.acquisition = "champ";
        h0.defaut = "70";
        addParamIn(h0);

        ParamIn Omega_m0 = new ParamIn();
        Omega_m0.type = "nombre";
        Omega_m0.label = "Omega_m0";
        Omega_m0.titre = "Contribution de la matire aujourd'hui";
        Omega_m0.unit = "densit critique";
        Omega_m0.acquisition = "champ";
        Omega_m0.defaut = "0.26";
        addParamIn(Omega_m0);

	ParamIn Omega_rad0 = new ParamIn();
        Omega_rad0.type = "nombre";
        Omega_rad0.label = "Omega_rad0";
        Omega_rad0.titre = "Temprature du rayonnement fossile ";
        Omega_rad0.unit = "K";
        Omega_rad0.acquisition = "champ";
        Omega_rad0.defaut = "2.75";
        addParamIn(Omega_rad0);

	ParamIn Omega_l0 = new ParamIn();
        Omega_l0.type = "nombre";
        Omega_l0.label = "Omega_l0";
        Omega_l0.titre = "Contribution de la constante cosmologique";
        Omega_l0.unit = "densit critique";
        Omega_l0.acquisition = "champ";
        Omega_l0.defaut = "0.74";
        addParamIn(Omega_l0);


	ParamIn fsn1a = new ParamIn();
        fsn1a.type = "string";
        fsn1a.label = "fsn1a";
        fsn1a.titre = "Donnes de SNe Ia";
        fsn1a.acquisition = "choix";
        String[] tchoix = {"SNLS06.dat", "Riess04.dat", "Daly03.dat"};
        fsn1a.choix = tchoix;
        fsn1a.defaut = "SNLS06.dat";
        addParamIn(fsn1a);


       
	ParamOut z = new ParamOut();
        z.type = "nombre";
        z.label = "z";
	z.titre = "z";
        addParamOut(z);

	ParamOut muz = new ParamOut();
	muz.type = "nombre";
	muz.label = "muz";
	muz.titre = "mu";
	addParamOut(muz);

	ParamOut zobs = new ParamOut();
        zobs.type = "nombre";
        zobs.label = "zobs";
	zobs.titre = "zobs";
        addParamOut(zobs);

	ParamOut muzobs = new ParamOut();
	muzobs.type = "nombre";
	muzobs.label = "muzobs";
	addParamOut(muzobs);

	ParamOut comment = new ParamOut();
	comment.type = "string";
	comment.label = "comment";
	addParamOut(comment);
	
	// options d'affichage
	// affichage "plotlive"
        Affichage p = new Affichage();
        p.type = "plotlive";
        p.titre = "Module de distance mu(z)";
	String[][] pparams = {{"z", "muz"},{"zobs", "muzobs"}};
        p.params = pparams;
        p.ajuster=true;
	p.carre=true;
	p.connecter=true;
	p.points="points";
	//p.fond="sn94d.gif";
	addAffichage(p);

	}

	// calcul live (utilis avec affichage "plotlive")
	// initialisation
	public void initCalculLive(ListeValeurs in) throws SimuException {

		h0 = in.lireDouble("h0");
		Omega_m0 = in.lireDouble("Omega_m0");
		Omega_l0 = in.lireDouble("Omega_l0");
		fsn1a = in.lireString("fsn1a");
		Omega_rad0 = in.lireDouble("Omega_rad0");
		Omega_rad0=4.1534e-5/h0/h0*100*100*Math.pow((Omega_rad0/2.725),4);

		z = 0;
		dl=0;
		muz=0;
		tempmuz=0;
		chi2=0;
		sum1=0;
		sum2=0;
		sum3=0;
		sum_int=0;
		idat=0;

		h0=h0*1.e3/mpc;
		Omega_k0=Omega_rad0+Omega_m0+Omega_l0-1.;

		// lecture du fichier
		String cheminf;
		cheminf = fsn1a;
		double[][] data = lireFichier(cheminf);
        
		zobs = data[0];
		muobs = data[1];
		sigobs = data[2];
		len=zobs.length;
	
	}


	// Calcul de zobs(j)  zobs(j+1)
	// Attention les donnes doivent tre ordonnes par ordre croissant de redshift
	public ListeValeurs calculLive() throws SimuException {
		if (z>=zobs[len-1]){
			System.err.println("chi2= "+chi2);
			return null; // fin du calcul
		}
		ListeValeurs out = new ListeValeurs();
		rombint(f2,z,zobs[idat],stop);
		z=zobs[idat];
		sum_int+=integral;
		
		if(Omega_k0>0) { // elliptic geometry
			dl=(1.+z)/Math.sqrt(Math.abs(Omega_k0))*Math.sin(sum_int*Math.sqrt(Math.abs(Omega_k0)))*3.e3/h0*1e5/mpc;
		}
		if(Omega_k0<0) { // hyperbolic geometry
			dl=sum_int*Math.sqrt(Math.abs(Omega_k0));
			dl=(1.+z)/Math.sqrt(Math.abs(Omega_k0))*(Math.exp(dl)-Math.exp(-dl))/2*3.e3/h0*1e5/mpc;
		}
		else{
			dl=sum_int*(z+1.)*3.e3/h0*1e5/mpc;
		}
		
		if(fsn1a == "Daly03.dat" ){
			muz=dl/(z+1.)/(3.e3/h0*1e5/mpc);
		}else{
			muz=25.+5.*Math.log(dl)/Math.log(10.);
		}

		if(fsn1a == "SNLS06.dat" ){
			sum1+=(muobs[idat]-muz)*(muobs[idat]-muz)/(sigobs[idat]*sigobs[idat]+0.13*0.13);
			sum2+=(muobs[idat]-muz)/(sigobs[idat]*sigobs[idat]+0.13*0.13);
			sum3+=1./(sigobs[idat]*sigobs[idat]+0.13*0.13);
		}
		if(fsn1a == "Riess04.dat" ){
			sum1+=(muobs[idat]-muz)*(muobs[idat]-muz)/(sigobs[idat]*sigobs[idat]);
			sum2+=(muobs[idat]-muz)/(sigobs[idat]*sigobs[idat]);
			sum3+=1./(sigobs[idat]*sigobs[idat]);
		}
		if(fsn1a == "Daly03.dat" ){
			sum1+=(muobs[idat]-muz)*(muobs[idat]-muz)/(sigobs[idat]*sigobs[idat]);
		}


		out.ajouterDouble("muz",muz);
		out.ajouterDouble("z",z);
		out.ajouterDouble("muzobs",muobs[idat]);
		out.ajouterDouble("zobs",zobs[idat]);
		idat++;
		if(fsn1a == "Daly03.dat" ){
			chi2=sum1;
			out.ajouterString("comment","chi^2 = "+Math.floor(10.*chi2)/10.+" pour "+len+" donnes");
		}
		else{
			chi2=sum1-sum2*sum2/sum3+Math.log(sum3/twopi)/Math.log(10.);
			out.ajouterString("comment","chi^2 marginalis sur H0 = "+Math.floor(10.*chi2)/10.+" pour "+len+" donnes");
		}


		return out;
	
	}
	

	//calcul du taux d'expansion en fonction de a
	public double hubble(double scale) {
		h=Omega_m0 * Math.pow(a0/scale,3.)+Omega_rad0 * Math.pow(a0/scale,4.)-Omega_k0 * Math.pow(a0/scale,2.)+Omega_l0;
		h=h0 * Math.sqrt(h);
		return h;
	}

	public void rombint(FUNC func,double x, double y, double tol) {
	//  Rombint returns the integral from a to b of f(x)dx using Romberg 
	// integration.
	//  The method converges provided that f(x) is continuous in (a,b).  
	//  The function f must be double precision and must be declared external 
	//  in the calling routine.  
	//  tol indicates the desired relative accuracy in the integral.

		double step,gmax,error,g0,fourj,g1,ftemp;
		int nint,i,jmax;
		int maxiter=20,maxj=5;
		double g[];
		g= new double[maxj+2];
	
		ftemp=0;
		step=0.5*(y-x);
		gmax=step*(func.fonction(y,ftemp)+func.fonction(x,ftemp));
		g[1]=gmax;
		nint=1;
		error=1.e20;
		i=0;
		g0=0;
		jmax=0;
		while (!(i>maxiter|(i>5 & Math.abs(error)< tol)) ){
		i++;
		//	Calculate next trapezoidal rule approximation to integral.

		g0=0;
		for(int k=0; k<nint; k++){
			g0=g0+func.fonction(x+(k+k-1)*step,ftemp);
		}
		g0=0.5*g[1]+step*g0;
		step=0.5*step;
		nint=nint+nint;
		jmax=Math.min(i,maxj);
		fourj=1;
		for(int j=0; j<jmax; j++){
		//	Use Richardson extrapolation.
			fourj=4.*fourj;
			g1=g0+(g0-g[j])/(fourj-1.);
			g[j]=g0;
			g0=g1;
		}
		if (Math.abs(g0)>tol) {
			error=1.-gmax/g0;
		}
		else{
			error=gmax;
		}
		gmax=g0;
		g[jmax+1]=g0;
		}
		integral=g0;
		/*
		if (i>maxiter & Math.abs(error)>tol) {
		System.err.println("Rombint failed to converge "+jmax);
		}
		*/
		}

	/**
	* lire un fichier ASCII 
	* avec le nombre d'lments sur la premire ligne
	* et " x  y  z" sur les suivantes -> t[0][i] = x; t[1][i] = y; t[2][i] = z;
	*/
	public double[][] lireFichier(String nomFichier) {
        	try {
		URL url = new URL(getDocumentBase(), nomFichier);
		BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream()));
		String ligne = in.readLine();
		int n = Integer.parseInt(ligne.trim());
		double[][] t = new double[3][n];
		for (int i=0; i<n && ligne != null; i++) {
                ligne = in.readLine();
                if (ligne != null) {
                    ligne = ligne.trim();
                    int is = ligne.indexOf(' ');
                    String sx = ligne.substring(0,is).trim();
                    ligne = ligne.substring(is+1).trim();
		    is = ligne.indexOf(' ');
		    String sy = ligne.substring(0,is).trim();
                    double x = (Double.valueOf(sx)).doubleValue();
                    double y = (Double.valueOf(sy)).doubleValue();    
		    String sz = ligne.substring(is+1).trim();
		    double z = (Double.valueOf(sz)).doubleValue();		
		    t[0][i] = x;
                    t[1][i] = y;
		    t[2][i] = z;
                }
            }
			
            return(t);
            } catch (MalformedURLException ex) {
            System.err.println(ex.getClass().getName() + " : " + ex.getMessage());
            return(null);
            } catch (NumberFormatException ex) {
            System.err.println(ex.getClass().getName() + " : " + ex.getMessage());
            return(null);
            } catch (IOException ex) {
            System.err.println(ex.getClass().getName() + " : " + ex.getMessage());
            return(null);
            }
        }
    
  public  interface FUNC {

    /**
    * Forme de la mthode fonction qui crit dans f le rsultat  de F(x)  intgrer
    *
    * @param x Argument x de F(x) donn par l'utilisateur
    * @param f Sortie de la mthode f = F(x).
    */

    public double  fonction(double x, double f);
	}

	public  class Fonction1 implements FUNC{

    /**
    * fonction  intgrer pour avoir le temps (1/(a*H))
    */
    public  double fonction(double x, double f) {

      f=Omega_m0 * Math.pow(a0/x,3.)+Omega_rad0 * Math.pow(a0/x,4.)-Omega_k0 * Math.pow(a0/x,2.)+Omega_l0;
	f=1/(h0 * Math.sqrt(f))/x*gyr;
	return f;
    }
	}
 
	public  class Fonction2 implements FUNC{

    /**
    * fonction  intgrer pour avoir la distance lumineuse (H0/H)
    */
    public  double fonction(double x, double f) {

    f=Omega_m0 * Math.pow((1.+x),3.)+Omega_rad0 * Math.pow((1.+x),4.)-Omega_k0 * Math.pow((1.+x),2.)+Omega_l0;
	f=1/Math.sqrt(f);
	return f;
    }
	}



}

